@@ -113,9 +113,11 @@ class Connection(object):
             ssl_cert=None,
             thrift_transport=None
     ):
-        username = username
-        password = password
-        self.database = database
+        self.__username = username
+        self.__password = password
+        self._database = database
+        self._session_id = None
+
         scheme = "e6xdb"
         # service_name = 'E6x'  # E6x  QueryExecutor
         service_name = 'QueryEngine'  # E6x  QueryExecutor
@@ -123,9 +125,8 @@ class Connection(object):
         if scheme != "e6xdb":
             raise ValueError("scheme is not e6xdb")
 
-        if password is None:
-            raise ValueError("Password should be set")
-
+        if not self.__username or not self.__password:
+            raise ValueError("username or password cannot be empty.")
         if port is None:
             port = 9000
         self._transport = TSocket.TSocket(host, port)
@@ -136,14 +137,31 @@ class Connection(object):
         self._client = QueryEngineService.Client(protocol)
         self._transport.open()
 
-        try:
-            authSession = self._client.authenticate(username, password)
-            if not authSession:
-                raise ValueError("invalid username or password")
-            # self._client.setSchema(database)
-        except Exception as e:
-            self._transport.close()
-            raise e
+    @property
+    def get_session_id(self):
+        """
+        To get the session id, if user is not authorised, first authenticate the user.
+        """
+        if not self._session_id:
+            try:
+                self._session_id = self._client.authenticate(self.__username, self.__password)
+                if not self._session_id:
+                    raise ValueError("Invalid credentials.")
+                # self._client.setSchema(database)
+            except Exception as e:
+                self._transport.close()
+                raise e
+        return self._session_id
+
+    def update_users(self, user_info):
+        self.client.updateUsers(userInfo=user_info)
+
+    def set_or_update_caches(self, is_cache_set: bool):
+        """
+        To enable to disable the caches.
+        :param is_cache_set: bool
+        """
+        self._client.setOrUnsetCaches(sessionId=self.get_session_id, bIsCacheSet=is_cache_set)
 
     def __enter__(self):
         """Transport should already be opened by __init__"""
@@ -160,32 +178,36 @@ class Connection(object):
         return self._transport.isOpen()
 
     def clear(self, query_id):
-        self._client.clear(query_id)
+        self._client.clear(sessionId=self.get_session_id, queryId=query_id)
+        self._session_id = None
 
     def reopen(self):
         self._transport.close()
         self._transport.open()
 
     def query_cancel(self, query_id):
-        self._client.cancelQuery(queryId=query_id)
+        self._client.cancelQuery(sessionId=self.get_session_id, queryId=query_id)
+
+    def dry_run(self, query):
+        return self._client.dryRun(sessionId=self.get_session_id, sSchema=self._database, sQueryString=query)
 
     def get_tables(self, database):
-        return self._client.getTables(schema=database)
+        return self._client.getTables(sessionId=self.get_session_id, schema=database)
 
     def getTables(self, database):
         return self.get_tables(database)
 
     def get_columns(self, database, table):
-        return self._client.getColumns(schema=database, table=table)
+        return self._client.getColumns(sessionId=self.get_session_id, schema=database, table=table)
 
     def getColumns(self, database, table):
         return self.get_columns(database, table)
 
     def get_schema_names(self):
-        return self._client.getSchemaNames()
+        return self._client.getSchemaNames(sessionId=self.get_session_id)
 
     def commit(self):
-        """Hive does not support transactions, so this does nothing."""
+        """We do not support transactions, so this does nothing."""
         pass
 
     def cursor(self, db_name=None):
@@ -219,7 +241,7 @@ class Cursor(DBAPICursor):
         self._query_id = None
         self._batch = list()
         self._rowcount = 0
-        self._database = self.connection.database if database is None else database
+        self._database = self.connection._database if database is None else database
 
     def _reset_state(self):
         """Reset state about the previous query in preparation for running another query"""
@@ -286,11 +308,11 @@ class Cursor(DBAPICursor):
         self._database = None
 
     def get_tables(self):
-        schema = self.connection.database
+        schema = self.connection._database
         return self.connection.get_tables(database=schema)
 
     def get_columns(self, table):
-        schema = self.connection.database
+        schema = self.connection._database
         return self.connection.get_columns(database=schema, table=table)
 
     def clear(self):
@@ -319,22 +341,19 @@ class Cursor(DBAPICursor):
 
         client = self.connection.client
         self._query_id = client.prepareStatement(
+            self.connection.get_session_id,
             self._database,
             sql
         )
-        client.executeStatement(self._query_id)
-
-        # _logger.info("getting result metadata")
+        client.executeStatement(self.connection.get_session_id, self._query_id)
         self.update_mete_data()
-        # _logger.info("finished populating metadata", self._query_id)
         return self._query_id
 
     def rowcount(self):
-        # _logger.info("fetch returning the rowcount from e6x itself")
         return self._rowcount
 
     def update_mete_data(self):
-        buffer = self.connection.client.getResultMetadata(self._query_id)
+        buffer = self.connection.client.getResultMetadata(self.connection.get_session_id, self._query_id)
         buffer = BytesIO(buffer)
         self._rowcount, self._query_columns_description = get_query_columns_info(buffer)
 
@@ -365,7 +384,7 @@ class Cursor(DBAPICursor):
     def fetch_batch(self):
         # _logger.debug("fetching next batch from e6data")
         client = self.connection.client
-        buffer = client.getNextResultBatch(self._query_id)
+        buffer = client.getNextResultBatch(self.connection.get_session_id, self._query_id)
         if not self._is_metadata_updated:
             self.update_mete_data()
             self._is_metadata_updated = True
@@ -404,7 +423,7 @@ class Cursor(DBAPICursor):
         # _logger.info("fetch One returning the batch itself which is limited by predefined no.of rows")
         rows_to_return = []
         client = self.connection.client
-        buffer = client.getNextResultRow(self._query_id)
+        buffer = client.getNextResultRow(self.connection.get_session_id, self._query_id)
         if not self._is_metadata_updated:
             self.update_mete_data()
             self._is_metadata_updated = True
@@ -446,6 +465,7 @@ def fetch_logs(self):
 class Error(Exception):
     pass
 
+
 #
 # Type Objects and Constructors
 #
@@ -454,15 +474,15 @@ for type_id in PRIMITIVE_TYPES:
     name = TypeId._VALUES_TO_NAMES[type_id]
     setattr(sys.modules[__name__], name, DBAPITypeObject([name]))
 
-
 if __name__ == '__main__':
-    host = '127.0.0.1'
+    host = '192.168.0.141'
+    database = 'tpcds_1000'
     e6x_connection = connect(
         host=host,
         port=9000,
         scheme="e6xdb",
         username="admin",
-        database="tpcds_ci",
+        database=database,
         auth=None,
         configuration=None,
         kerberos_service_name=None,
@@ -472,12 +492,28 @@ if __name__ == '__main__':
         thrift_transport=None
     )
     print('Connected.')
+
+    query = 'select * from date_dim limit 2'
+    # res = e6x_connection.dry_run(query)
+    # print(res)
+    # exit()
+    # print(e6x_connection.get_schema_names())
+    # exit()
+    cursor = e6x_connection.cursor()
+    qid = cursor.execute(query)
+    print('qid', qid)
+    all_records = cursor.fetchone()
+    print(all_records)
+    cursor.clear()
+    e6x_connection.close()
+    exit()
+
     import time
 
+    # print(e6x_connection.dry_run('tpcds_1000', query))
     start = time.time()
     cursor = e6x_connection.cursor()
 
-    query = 'select * from date_dim limit 2'
     query_id = cursor.execute(query)
     print('query_id', query_id)
     all_records = cursor.fetchall()
@@ -485,6 +521,7 @@ if __name__ == '__main__':
     for record in all_records:
         print(all_records)
     print(cursor.explain_analyse())
+    cursor.clear()
     e6x_connection.close()
 #
 #     e6x_connection.reopen()
